@@ -2,16 +2,22 @@
 -- |
 -- Module      :  Language.Haskell.ParseMonad
 -- Copyright   :  (c) The GHC Team, 1997-2000
--- License     :  BSD-style (see the file libraries/base/LICENSE)
+-- License     :  BSD-3-Clause
 --
--- Maintainer  :  libraries@haskell.org
--- Stability   :  experimental
+-- Maintainer  :  Andreas Abel
+-- Stability   :  stable
 -- Portability :  portable
 --
 -- Monads for the Haskell parser and lexer.
 --
 -----------------------------------------------------------------------------
 
+{-# LANGUAGE CPP #-}
+
+#if __GLASGOW_HASKELL__ >= 902
+{-# OPTIONS_GHC -fno-warn-incomplete-uni-patterns #-}
+#endif
+
 module Language.Haskell.ParseMonad(
                 -- * Parsing
                 P, ParseResult(..), atSrcLoc, LexContext(..),
@@ -48,6 +54,7 @@ instance App.Applicative ParseResult where
   ParseFailed loc msg <*> _ = ParseFailed loc msg
 
 instance Monad ParseResult where
+  return = pure
   ParseOk x           >>= f = f x
   ParseFailed loc msg >>= _ = ParseFailed loc msg
 
@@ -62,6 +69,7 @@ instance Monoid m => Semi.Semigroup (ParseResult m) where
 
 instance Monoid m => Monoid (ParseResult m) where
   mempty = ParseOk mempty
+  mappend = (<>)
 
 -- internal version
 data ParseStatus a = Ok ParseState a | Failed SrcLoc String
@@ -127,10 +135,14 @@ instance Applicative P where
         (<*>) = ap
 
 instance Monad P where
+        return = pure
         P m >>= k = P $ \i x y l s mode ->
                 case m i x y l s mode of
                     Failed loc msg -> Failed loc msg
                     Ok s' a        -> runP (k a) i x y l s' mode
+#if !(MIN_VERSION_base(4,13,0))
+        fail = Fail.fail
+#endif
 
 -- | @since 1.0.3.0
 instance Fail.MonadFail P where
@@ -184,8 +196,12 @@ instance Applicative (Lex r) where
         Lex v *> Lex w = Lex $ \k -> v (\_ -> w k)
 
 instance Monad (Lex r) where
+        return = pure
         Lex v >>= f = Lex $ \k -> v (\a -> runL (f a) k)
         (>>) = (*>)
+#if !(MIN_VERSION_base(4,13,0))
+        fail = Fail.fail
+#endif
 
 -- | @since 1.0.3.0
 instance Fail.MonadFail (Lex r) where
